# ###############
# # DESCRIPTION #
# ###############

# This script makes an ordered list from a newline-separated string.
# For more information run it with the -h argument: "make-ordered-list.py -h".

# ##################
# # Import Modules #
# ##################

import os
import sys
import getopt
import operator

# #########################################
# # Define Global Constants And Variables #
# #########################################

SCRIPT_NAME =                str(sys.argv[0][sys.argv[0].rfind("\\")+1:]),                         # Name of the script (without path)

MAN =                        [
                             ("NAME",            "This script makes an ordered list from a newline-separated string."),
														 ("VER",             "1.0"),
                             ("SYNOPSIS",        "[OPTION]... <input>"),
                             ("DESCRIPTION",     ""),
                             ("AUTHOR",          "Written by Adam Pócs"),
                             ("REPORTING BUGS",  "Report bugs to royaljerry@gmail.com"),
                             ("COPYRIGHT",       "Copyright (C) 2007 Adam Pócs; this is free software.")
                             ]
OPTIONS_SHORT =              [
                             ("h",               "Print this message and exit."),
                             ("f",               "Take input as file."),
                             ("s",               "Trim leading spaces."),
                             ("z",               "Fill numbers with zero digits."),
                             ("a",               "Align numbers left."),
                             ("p",               "Preserve maximum width for numbers."),
                             ("d <character>",   "Left delimiter is <character>."),
                             ("D <character>",   "Right delimiter is <character>."),
                             ("t <number>",      "Distance between numbers and text is <number> (excl. right delimiter)."),
                             ("n <number>",      "Distance between delimiters and numbers is <number>."),
                             ("v",               "Verbose mode.")
                             ]
OPTIONS_LONG =               [
                             ("regular",         (
                                                 "Trim leading spaces, fill numbers with zero digits, no left",
                                                 "or right delimiters, distance between numbers and text is 1,",
                                                 "not verbose mode (same as -sz)."
                                                 )),
                             ("right-sep",       (
                                                 "Trim leading spaces, preserve maximum width for numbers,",
                                                 "right delimiter is |, distance between numbers and text is 1,",
                                                 "distance between right delimiter and numbers is 1,",
                                                 "not verbose mode (same as -sp -D | -n 1)."
                                                 )),
                             ]

TAB_0 =                      7                                                                     # Indent of the usage sytax descriptions before arguments when requestng help
TAB_1 =                      20                                                                    # Indent of the usage sytax descriptions after arguments when requestng help
INPUT_STRING =               sys.argv[len(sys.argv)-1]                                             # Input string reference
INPUT_STRING_OBJ =           ""                                                                    # Input string reference as object
INPUT_STRING_LINES =         []                                                                    # List of input string lines
OUTPUT_STRING =              ""                                                                    # Output string
OUTPUT_LINES =               []                                                                    # List of output string lines

O_HELP =                     False                                                                 # Print this message and exit
O_FILE =                     False                                                                 # Take input as file
O_TRIM =                     False                                                                 # Trim leading spaces
O_ZERO =                     False                                                                 # Fill numbers with zero digits
O_LEFT =                     False                                                                 # Align numbers left
O_PRESERVE =                 False                                                                 # Preserve maximum width for numbers
O_DEL_L =                    ""                                                                    # Left delimiter is <character>
O_DEL_R =                    ""                                                                    # Right delimiter is <character>
O_DIST_TEXT =                1                                                                     # Distance between numbers and text is <number> (excl. right delimiter)
O_DIST_NUM =                 0                                                                     # Distance between delimiters and numbers is <number>
O_VERBOSE =                  False                                                                 # Verbose mode

# #############
# # Functions #
# #############

def printUsage(sName, man, oShort, oLong, indent0, indent1):
	# Init vars for the text to be printed
	cPrint = ""
	fChar = " "
	# Print sections and their descriptions
	for i in range(len(man)):
		nMan = man[i][0]
		if nMan == "NAME": dMan = sName + ": " + man[i][1]
		elif nMan == "SYNOPSIS": dMan = sName + " " + man[i][1]
		else: dMan = man[i][1]
		if nMan == "DESCRIPTION":
			cPrint += nMan
			# Print short descriptions
			for opt in range(len(oShort)):
				cPrint += "\n\n"
				cLine = str(fChar*TAB_0) + "-" + oShort[opt][0].ljust(indent1, " ") + oShort[opt][1]
				cPrint += cLine
			# Print long descriptions
			for opt in range(len(oLong)):
				cPrint += "\n\n"
				cLine = ""
				for desc in range(len(oLong[opt][1])):
					if desc == 0: cLine = str(fChar*TAB_0) + "--" + oLong[opt][0].ljust(indent1-1, " ") + oLong[opt][1][desc] + "\n"
					elif desc < len(oLong[opt][1])-1: cLine = str(fChar*((indent1+TAB_0)+1)) + oLong[opt][1][desc] + "\n"
					else: cLine = str(fChar*((indent1+TAB_0)+1)) + oLong[opt][1][desc]
					cPrint += cLine
		else:
			cPrint += nMan
			cPrint += "\n\n"
			cPrint += str(fChar*indent0) + dMan
		cPrint += "\n\n\n"
	print (cPrint)

def parseOptions():
	# Collect all possible short option names
	cOptsShort = ""
	for i in range(len(OPTIONS_SHORT)):
		if len(OPTIONS_SHORT[i][0]) == 1: cOptsShort += OPTIONS_SHORT[i][0] + ","
		else: cOptsShort += OPTIONS_SHORT[i][0][0:1] + ":,"
	cOptsShort = cOptsShort.rstrip(",")
	# Collect all possible long option names
	cOptsLong = []
	for i in range(len(OPTIONS_LONG)): cOptsLong.append(OPTIONS_LONG[i][0])
	try:
		# Get current options
		opts, args = getopt.getopt(sys.argv[1:], cOptsShort, cOptsLong)
	except getopt.GetoptError:
		# If something went wrong, print error, help information and exit
		print ("\nSomething went wrong...\n")
		printUsage(globals()["SCRIPT_NAME"][0], globals()["MAN"], globals()["OPTIONS_SHORT"], globals()["OPTIONS_LONG"], globals()["TAB_0"], globals()["TAB_1"])
		sys.exit(2)
	
	# Exit if no input specified
	if len(sys.argv) == 1:
		print ("\n" + str(globals()["SCRIPT_NAME"][0]) + " needs at least one argument...\n")
		printUsage(globals()["SCRIPT_NAME"][0], globals()["MAN"], globals()["OPTIONS_SHORT"], globals()["OPTIONS_LONG"], globals()["TAB_0"], globals()["TAB_1"])
		sys.exit(0)
	
	if O_VERBOSE: print ("")

	# Parse long options
	for op, ar in opts:
		if op == "--regular":
			globals()["O_TRIM"] =                      True
			globals()["O_ZERO"] =                      True
		if op == "--right-sep":
			globals()["O_TRIM"] =                      True
			globals()["O_PRESERVE"] =                  True
			globals()["O_DEL_R"] =                     "|"
			globals()["O_DIST_NUM"] =                  1

	# Parse short options
	for op, ar in opts:
		if op == "-h": globals()["O_HELP"] =         True
		if op == "-f": globals()["O_FILE"] =         True
		if op == "-s": globals()["O_TRIM"] =         True
		if op == "-z": globals()["O_ZERO"] =         True
		if op == "-a": globals()["O_LEFT"] =         True
		if op == "-p": globals()["O_PRESERVE"] =     True
		if op == "-d": globals()["O_DEL_L"] =        ar
		if op == "-D": globals()["O_DEL_R"] =        ar
		if op == "-t": globals()["O_DIST_TEXT"] =    int(ar)
		if op == "-n": globals()["O_DIST_NUM"] =     int(ar)
		if op == "-v":
			globals()["O_VERBOSE"] = True
			print ("\nEntering Verbose Mode...\n")

def trimLeading(inputLines):
	oLines = []
	for line in inputLines: oLines.append(line.lstrip())
	return (oLines)	

def makeList(inputLines):
	fChar = " "
	cSep = O_DIST_NUM*fChar
	mNum = len(inputLines)+1
	mNumL = len(str(mNum))
	cL = ""
	cR = ""
	if O_DEL_L != "": cL = O_DEL_L + cSep
	if O_DEL_R != "": cR = cSep + O_DEL_R
	oLines = []
	for i in range(len(inputLines)):
		cNum = str(i+1)
		if O_ZERO: cNum = cNum.zfill(mNumL)
		if O_LEFT == False: cNum = cNum.rjust(mNumL)
		if (O_LEFT == True) and (O_PRESERVE == True): cNum = cNum.ljust(mNumL)
		cLineNum = cL + cNum + cR
		cLineCont = inputLines[i].rstrip("\n")
		cLine = (cLineNum, cLineCont)
		oLines.append(cLine)
	return (oLines)

def main():
	fChar = " "

	# Parse options
	parseOptions()

	if O_VERBOSE:
		print ("Print this message and exit (-h): ".ljust(80, ".") + str(O_HELP))
		print ("Take input as file (-f): ".ljust(80, ".") + str(O_FILE))
		print ("Trim leading spaces (-s): ".ljust(80, ".") + str(O_TRIM))
		print ("Fill numbers with zero digits (-z): ".ljust(80, ".") + str(O_ZERO))
		print ("Align numbers left (-a): ".ljust(80, ".") + str(O_LEFT))
		print ("Preserve maximum width for numbers (-p): ".ljust(80, ".") + str(O_PRESERVE))
		print ("Left delimiter is <character> (-d): ".ljust(80, ".") + str(O_DEL_L))
		print ("Right delimiter is <character> (-D): ".ljust(80, ".") + str(O_DEL_R))
		print ("Distance between numbers and text is <number> (excl. right delimiter) (-t): ".ljust(80, ".") + str(O_DIST_TEXT))
		print ("Distance between delimiters and numbers is <number> (-n): ".ljust(80, ".") + str(O_DIST_NUM))
		print ("Verbose mode (-v): ".ljust(80, ".") + str(O_VERBOSE))

	# Exit if help requested
	if O_HELP:
		printUsage(globals()["SCRIPT_NAME"][0], globals()["MAN"], globals()["OPTIONS_SHORT"], globals()["OPTIONS_LONG"], globals()["TAB_0"], globals()["TAB_1"])
		sys.exit(0)
	
	# Define input
	if O_FILE:
		try:
			INPUT_STRING_OBJ = open(INPUT_STRING)
			INPUT_STRING_LINES = INPUT_STRING_OBJ.readlines()
		except:
			print ("Cannot open " + INPUT_STRING)
			sys.exit(2)
	else:
		INPUT_STRING_OBJ = INPUT_STRING
		INPUT_STRING_LINES = INPUT_STRING_OBJ.splitlines()
	
	# Trim leading spaces if needed
	if O_TRIM: INPUT_STRING_LINES = trimLeading(INPUT_STRING_LINES)

	# Run Filter
	resultLines = makeList(INPUT_STRING_LINES)

	# Print result
	if O_VERBOSE:
		print ("\n\n##########")
		print ("#        #")
		print ("# RESULT #")
		print ("#        #")
		print ("##########\n")
	for i in range(len(resultLines)):
		cNum = resultLines[i][0]
		cSep = str(fChar*O_DIST_TEXT)
		cCon = resultLines[i][1]
		cLine = str(cNum) + cSep + str(cCon)
		cPrint = str(cLine)
		print (cPrint)
	if O_VERBOSE: print ("")

# ########
# # MAIN #
# ########

main ()

# TODO:
#
# Error handling (what if we pass "C" as number of spaces, etc.)
# Ability to start line numbers from N
# Ability to set numbers' width directly to N (can be longer than the greatest number's width)
# Ability to iterate numbers with N
# Ability to make ordered list based on a predefined list (e.g. Months, Days, etc.)
# Ability to make unordered list
# Ability to insert n empty lines between result lines
# Ability to trim all leading blank characters (incl. tabs, and other white spaces)
